@@ -19,6 +19,7 @@ package org.apache.hadoop.hbase.client;
 
 import java.io.IOException;
 import java.io.InterruptedIOException;
+import java.net.SocketTimeoutException;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
@@ -33,8 +34,8 @@ import java.util.concurrent.ExecutorService;
 import java.util.concurrent.RejectedExecutionException;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicLong;
-import org.apache.hadoop.hbase.CallQueueTooBigException;
 import org.apache.hadoop.hbase.DoNotRetryIOException;
+import org.apache.hadoop.hbase.HBaseServerException;
 import org.apache.hadoop.hbase.HConstants;
 import org.apache.hadoop.hbase.HRegionLocation;
 import org.apache.hadoop.hbase.RegionLocations;
@@ -44,10 +45,8 @@ import org.apache.hadoop.hbase.TableName;
 import org.apache.hadoop.hbase.client.backoff.ServerStatistics;
 import org.apache.hadoop.hbase.client.coprocessor.Batch;
 import org.apache.hadoop.hbase.exceptions.ClientExceptionsUtil;
-import org.apache.hadoop.hbase.trace.TraceUtil;
 import org.apache.hadoop.hbase.util.Bytes;
 import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
-import org.apache.htrace.core.Tracer;
 import org.apache.yetus.audience.InterfaceAudience;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -410,6 +409,28 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
     return new SingleServerRequestRunnable(multiAction, numAttempt, server, callsInProgress);
   }
 
+  /**
+   * Some checked calls send a callable with their own tracker. This method checks the operation
+   * timeout against the appropriate tracker, or returns false if no tracker.
+   */
+  private boolean isOperationTimeoutExceeded() {
+    RetryingTimeTracker currentTracker;
+    if (tracker != null) {
+      currentTracker = tracker;
+    } else if (currentCallable != null && currentCallable.getTracker() != null) {
+      currentTracker = currentCallable.getTracker();
+    } else {
+      return false;
+    }
+
+    // no-op if already started, this is just to ensure it was initialized (usually true)
+    currentTracker.start();
+
+    // return value of 1 is special to mean exceeded, to differentiate from 0
+    // which is no timeout. see implementation of getRemainingTime
+    return currentTracker.getRemainingTime(operationTimeout) == 1;
+  }
+
   /**
    * Group a list of actions per region servers, and send them.
    * @param currentActions - the list of row to submit
@@ -421,6 +442,27 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
     boolean isReplica = false;
     List<Action> unknownReplicaActions = null;
     for (Action action : currentActions) {
+      if (isOperationTimeoutExceeded()) {
+        String message = numAttempt == 1
+          ? "Operation timeout exceeded during resolution of region locations, "
+            + "prior to executing any actions."
+          : "Operation timeout exceeded during re-resolution of region locations on retry "
+            + (numAttempt - 1) + ".";
+
+        message += " Meta may be slow or operation timeout too short for batch size or retries.";
+        OperationTimeoutExceededException exception =
+          new OperationTimeoutExceededException(message);
+
+        // Clear any actions we already resolved, because none will have been executed yet
+        // We are going to fail all passed actions because there's no way we can execute any
+        // if operation timeout is exceeded.
+        actionsByServer.clear();
+        for (Action actionToFail : currentActions) {
+          manageLocationError(actionToFail, exception);
+        }
+        return;
+      }
+
       RegionLocations locs = findAllLocationsOrFail(action, true);
       if (locs == null) continue;
       boolean isReplicaAction = !RegionReplicaUtil.isDefaultReplica(action.getReplicaId());
@@ -549,7 +591,7 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
           runnable.run();
         } else {
           try {
-            pool.submit(runnable);
+            pool.execute(runnable);
           } catch (Throwable t) {
             if (t instanceof RejectedExecutionException) {
               // This should never happen. But as the pool is provided by the end user,
@@ -576,6 +618,7 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
     }
   }
 
+  @SuppressWarnings("MixedMutabilityReturnType")
   private Collection<? extends Runnable> getNewMultiActionRunnable(ServerName server,
     MultiAction multiAction, int numAttempt) {
     // no stats to manage, just do the standard action
@@ -586,13 +629,9 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
       asyncProcess.incTaskCounters(multiAction.getRegions(), server);
       SingleServerRequestRunnable runnable =
         createSingleServerRequest(multiAction, numAttempt, server, callsInProgress);
-      Tracer tracer = Tracer.curThreadTracer();
 
-      if (tracer == null) {
-        return Collections.singletonList(runnable);
-      } else {
-        return Collections.singletonList(tracer.wrap(runnable, "AsyncProcess.sendMultiAction"));
-      }
+      // remove trace for runnable because HBASE-25373 and OpenTelemetry do not cover TraceRunnable
+      return Collections.singletonList(runnable);
     }
 
     // group the actions by the amount of delay
@@ -612,13 +651,11 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
     List<Runnable> toReturn = new ArrayList<>(actions.size());
     for (DelayingRunner runner : actions.values()) {
       asyncProcess.incTaskCounters(runner.getActions().getRegions(), server);
-      String traceText = "AsyncProcess.sendMultiAction";
       Runnable runnable =
         createSingleServerRequest(runner.getActions(), numAttempt, server, callsInProgress);
       // use a delay runner only if we need to sleep for some time
       if (runner.getSleepTime() > 0) {
         runner.setRunner(runnable);
-        traceText = "AsyncProcess.clientBackoff.sendMultiAction";
         runnable = runner;
         if (asyncProcess.connection.getConnectionMetrics() != null) {
           asyncProcess.connection.getConnectionMetrics()
@@ -629,7 +666,7 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
           asyncProcess.connection.getConnectionMetrics().incrNormalRunners();
         }
       }
-      runnable = TraceUtil.wrap(runnable, traceText);
+      // remove trace for runnable because HBASE-25373 and OpenTelemetry do not cover TraceRunnable
       toReturn.add(runnable);
 
     }
@@ -662,7 +699,7 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
       // Start the thread that may kick off replica gets.
       // TODO: we could do it on the same thread, but it's a user thread, might be a bad idea.
       try {
-        pool.submit(replicaRunnable);
+        pool.execute(replicaRunnable);
       } catch (RejectedExecutionException ree) {
         LOG.warn("id=" + asyncProcess.id + " replica task rejected by pool; no replica calls", ree);
       }
@@ -781,12 +818,21 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
     long backOffTime;
     if (retryImmediately) {
       backOffTime = 0;
-    } else if (throwable instanceof CallQueueTooBigException) {
-      // Give a special check on CQTBE, see #HBASE-17114
-      backOffTime = errorsByServer.calculateBackoffTime(oldServer, asyncProcess.pauseForCQTBE);
+    } else if (HBaseServerException.isServerOverloaded(throwable)) {
+      // Give a special check when encountering an exception indicating the server is overloaded.
+      // see #HBASE-17114 and HBASE-26807
+      backOffTime = errorsByServer.calculateBackoffTime(oldServer,
+        asyncProcess.connectionConfiguration.getPauseMillisForServerOverloaded());
     } else {
-      backOffTime = errorsByServer.calculateBackoffTime(oldServer, asyncProcess.pause);
+      backOffTime = errorsByServer.calculateBackoffTime(oldServer,
+        asyncProcess.connectionConfiguration.getPauseMillis());
     }
+
+    MetricsConnection metrics = asyncProcess.connection.getConnectionMetrics();
+    if (metrics != null && HBaseServerException.isServerOverloaded(throwable)) {
+      metrics.incrementServerOverloadedBackoffTime(backOffTime, TimeUnit.MILLISECONDS);
+    }
+
     if (numAttempt > asyncProcess.startLogErrorsCnt) {
       // We use this value to have some logs when we have multiple failures, but not too many
       // logs, as errors are to be expected when a region moves, splits and so on
@@ -811,6 +857,7 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
   private void logNoResubmit(ServerName oldServer, int numAttempt, int failureCount,
     Throwable throwable, int failed, int stopped) {
     if (failureCount != 0 || numAttempt > asyncProcess.startLogErrorsCnt + 1) {
+      @SuppressWarnings("JavaUtilDate")
       String timeStr = new Date(errorsByServer.getStartTrackingTime()).toString();
       String logMessage = createLog(numAttempt, failureCount, 0, oldServer, throwable, -1, false,
         timeStr, failed, stopped);
@@ -942,7 +989,7 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
   }
 
   private void cleanServerCache(ServerName server, Throwable regionException) {
-    if (ClientExceptionsUtil.isMetaClearingException(regionException)) {
+    if (tableName == null && ClientExceptionsUtil.isMetaClearingException(regionException)) {
       // We want to make sure to clear the cache in case there were location-related exceptions.
       // We don't to clear the cache for every possible exception that comes through, however.
       MetricsConnection metrics = asyncProcess.connection.getConnectionMetrics();
@@ -1000,14 +1047,14 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
     if (result == null) {
       throw new RuntimeException("Result cannot be null");
     }
-    ReplicaResultState state = null;
     boolean isStale = !RegionReplicaUtil.isDefaultReplica(action.getReplicaId());
     int index = action.getOriginalIndex();
     if (results == null) {
       decActionCounter(index);
       return; // Simple case, no replica requests.
     }
-    state = trySetResultSimple(index, action.getAction(), false, result, null, isStale);
+    ReplicaResultState state =
+      trySetResultSimple(index, action.getAction(), false, result, null, isStale);
     if (state == null) {
       return; // Simple case, no replica requests.
     }
@@ -1040,7 +1087,6 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
    * @param server    The source server.
    */
   private void setError(int index, Row row, Throwable throwable, ServerName server) {
-    ReplicaResultState state = null;
     if (results == null) {
       // Note that we currently cannot have replica requests with null results. So it shouldn't
       // happen that multiple replica calls will call dAC for same actions with results == null.
@@ -1049,7 +1095,7 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
       decActionCounter(index);
       return; // Simple case, no replica requests.
     }
-    state = trySetResultSimple(index, row, true, throwable, server, false);
+    ReplicaResultState state = trySetResultSimple(index, row, true, throwable, server, false);
     if (state == null) {
       return; // Simple case, no replica requests.
     }
@@ -1180,7 +1226,17 @@ class AsyncRequestFutureImpl<CResult> implements AsyncRequestFuture {
   @Override
   public void waitUntilDone() throws InterruptedIOException {
     try {
-      waitUntilDone(Long.MAX_VALUE);
+      if (this.operationTimeout > 0) {
+        // the worker thread maybe over by some exception without decrement the actionsInProgress,
+        // then the guarantee of operationTimeout will be broken, so we should set cutoff to avoid
+        // stuck here forever
+        long cutoff = (EnvironmentEdgeManager.currentTime() + this.operationTimeout) * 1000L;
+        if (!waitUntilDone(cutoff)) {
+          throw new SocketTimeoutException("time out before the actionsInProgress changed to zero");
+        }
+      } else {
+        waitUntilDone(Long.MAX_VALUE);
+      }
     } catch (InterruptedException iex) {
       throw new InterruptedIOException(iex.getMessage());
     } finally {
