@@ -24,11 +24,9 @@ import java.util.ArrayDeque;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
-import java.util.Comparator;
 import java.util.Deque;
 import java.util.HashSet;
 import java.util.List;
-import java.util.PriorityQueue;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.CopyOnWriteArrayList;
@@ -48,8 +46,10 @@ import org.apache.hadoop.hbase.procedure2.Procedure.LockState;
 import org.apache.hadoop.hbase.procedure2.store.ProcedureStore;
 import org.apache.hadoop.hbase.procedure2.store.ProcedureStore.ProcedureIterator;
 import org.apache.hadoop.hbase.procedure2.store.ProcedureStore.ProcedureStoreListener;
+import org.apache.hadoop.hbase.procedure2.trace.ProcedureSpanBuilder;
 import org.apache.hadoop.hbase.procedure2.util.StringUtils;
 import org.apache.hadoop.hbase.security.User;
+import org.apache.hadoop.hbase.trace.TraceUtil;
 import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.hadoop.hbase.util.IdLock;
 import org.apache.hadoop.hbase.util.NonceKey;
@@ -117,9 +117,6 @@ public class ProcedureExecutor<TEnvironment> {
     protected volatile boolean killAfterStoreUpdate = false;
     protected volatile boolean toggleKillAfterStoreUpdate = false;
 
-    protected volatile boolean killBeforeStoreUpdateInRollback = false;
-    protected volatile boolean toggleKillBeforeStoreUpdateInRollback = false;
-
     protected boolean shouldKillBeforeStoreUpdate() {
       final boolean kill = this.killBeforeStoreUpdate;
       if (this.toggleKillBeforeStoreUpdate) {
@@ -151,16 +148,6 @@ public class ProcedureExecutor<TEnvironment> {
     protected boolean shouldKillAfterStoreUpdate(final boolean isSuspended) {
       return (isSuspended && !killIfSuspended) ? false : shouldKillAfterStoreUpdate();
     }
-
-    protected boolean shouldKillBeforeStoreUpdateInRollback() {
-      final boolean kill = this.killBeforeStoreUpdateInRollback;
-      if (this.toggleKillBeforeStoreUpdateInRollback) {
-        this.killBeforeStoreUpdateInRollback = !kill;
-        LOG.warn("Toggle KILL before store update in rollback to: "
-          + this.killBeforeStoreUpdateInRollback);
-      }
-      return kill;
-    }
   }
 
   public interface ProcedureExecutorListener {
@@ -293,7 +280,7 @@ public class ProcedureExecutor<TEnvironment> {
         }
         long evictTtl = conf.getInt(EVICT_TTL_CONF_KEY, DEFAULT_EVICT_TTL);
         long evictAckTtl = conf.getInt(EVICT_ACKED_TTL_CONF_KEY, DEFAULT_ACKED_EVICT_TTL);
-        if (retainer.isExpired(System.currentTimeMillis(), evictTtl, evictAckTtl)) {
+        if (retainer.isExpired(EnvironmentEdgeManager.currentTime(), evictTtl, evictAckTtl)) {
           LOG.debug("Procedure {} has already been finished and expired, skip force updating",
             procId);
           return;
@@ -407,108 +394,6 @@ public class ProcedureExecutor<TEnvironment> {
     });
   }
 
-  private void initializeStacks(ProcedureIterator procIter,
-    List<Procedure<TEnvironment>> runnableList, List<Procedure<TEnvironment>> failedList,
-    List<Procedure<TEnvironment>> waitingList, List<Procedure<TEnvironment>> waitingTimeoutList)
-    throws IOException {
-    procIter.reset();
-    while (procIter.hasNext()) {
-      if (procIter.isNextFinished()) {
-        procIter.skipNext();
-        continue;
-      }
-
-      @SuppressWarnings("unchecked")
-      Procedure<TEnvironment> proc = procIter.next();
-      assert !(proc.isFinished() && !proc.hasParent()) : "unexpected completed proc=" + proc;
-      LOG.debug("Loading {}", proc);
-      Long rootProcId = getRootProcedureId(proc);
-      // The orphan procedures will be passed to handleCorrupted, so add an assert here
-      assert rootProcId != null;
-
-      if (proc.hasParent()) {
-        Procedure<TEnvironment> parent = procedures.get(proc.getParentProcId());
-        if (parent != null && !proc.isFinished()) {
-          parent.incChildrenLatch();
-        }
-      }
-
-      RootProcedureState<TEnvironment> procStack = rollbackStack.get(rootProcId);
-      procStack.loadStack(proc);
-
-      proc.setRootProcId(rootProcId);
-      switch (proc.getState()) {
-        case RUNNABLE:
-          runnableList.add(proc);
-          break;
-        case WAITING:
-          waitingList.add(proc);
-          break;
-        case WAITING_TIMEOUT:
-          waitingTimeoutList.add(proc);
-          break;
-        case FAILED:
-          failedList.add(proc);
-          break;
-        case ROLLEDBACK:
-        case INITIALIZING:
-          String msg = "Unexpected " + proc.getState() + " state for " + proc;
-          LOG.error(msg);
-          throw new UnsupportedOperationException(msg);
-        default:
-          break;
-      }
-    }
-    rollbackStack.forEach((rootProcId, procStack) -> {
-      if (procStack.getSubproceduresStack() != null) {
-        // if we have already record some stack ids, it means we support rollback
-        procStack.setRollbackSupported(true);
-      } else {
-        // otherwise, test the root procedure to see if we support rollback
-        procStack.setRollbackSupported(procedures.get(rootProcId).isRollbackSupported());
-      }
-    });
-  }
-
-  private void processWaitingProcedures(List<Procedure<TEnvironment>> waitingList,
-    List<Procedure<TEnvironment>> runnableList) {
-    waitingList.forEach(proc -> {
-      if (!proc.hasChildren()) {
-        // Normally, WAITING procedures should be waken by its children. But, there is a case that,
-        // all the children are successful and before they can wake up their parent procedure, the
-        // master was killed. So, during recovering the procedures from ProcedureWal, its children
-        // are not loaded because of their SUCCESS state. So we need to continue to run this WAITING
-        // procedure. But before executing, we need to set its state to RUNNABLE, otherwise, a
-        // exception will throw:
-        // Preconditions.checkArgument(procedure.getState() == ProcedureState.RUNNABLE,
-        // "NOT RUNNABLE! " + procedure.toString());
-        proc.setState(ProcedureState.RUNNABLE);
-        runnableList.add(proc);
-      } else {
-        proc.afterReplay(getEnvironment());
-      }
-    });
-  }
-
-  private void processWaitingTimeoutProcedures(List<Procedure<TEnvironment>> waitingTimeoutList) {
-    waitingTimeoutList.forEach(proc -> {
-      proc.afterReplay(getEnvironment());
-      timeoutExecutor.add(proc);
-    });
-  }
-
-  private void pushProceduresAfterLoad(List<Procedure<TEnvironment>> runnableList,
-    List<Procedure<TEnvironment>> failedList) {
-    failedList.forEach(scheduler::addBack);
-    runnableList.forEach(p -> {
-      p.afterReplay(getEnvironment());
-      if (!p.hasParent()) {
-        sendProcedureLoadedNotification(p.getProcId());
-      }
-      scheduler.addBack(p);
-    });
-  }
-
   private void loadProcedures(ProcedureIterator procIter) throws IOException {
     // 1. Build the rollback stack
     int runnableCount = 0;
@@ -571,20 +456,90 @@ public class ProcedureExecutor<TEnvironment> {
     List<Procedure<TEnvironment>> failedList = new ArrayList<>(failedCount);
     List<Procedure<TEnvironment>> waitingList = new ArrayList<>(waitingCount);
     List<Procedure<TEnvironment>> waitingTimeoutList = new ArrayList<>(waitingTimeoutCount);
+    procIter.reset();
+    while (procIter.hasNext()) {
+      if (procIter.isNextFinished()) {
+        procIter.skipNext();
+        continue;
+      }
 
-    initializeStacks(procIter, runnableList, failedList, waitingList, waitingTimeoutList);
+      @SuppressWarnings("unchecked")
+      Procedure<TEnvironment> proc = procIter.next();
+      assert !(proc.isFinished() && !proc.hasParent()) : "unexpected completed proc=" + proc;
+      LOG.debug("Loading {}", proc);
+      Long rootProcId = getRootProcedureId(proc);
+      // The orphan procedures will be passed to handleCorrupted, so add an assert here
+      assert rootProcId != null;
+
+      if (proc.hasParent()) {
+        Procedure<TEnvironment> parent = procedures.get(proc.getParentProcId());
+        if (parent != null && !proc.isFinished()) {
+          parent.incChildrenLatch();
+        }
+      }
+
+      RootProcedureState<TEnvironment> procStack = rollbackStack.get(rootProcId);
+      procStack.loadStack(proc);
+
+      proc.setRootProcId(rootProcId);
+      switch (proc.getState()) {
+        case RUNNABLE:
+          runnableList.add(proc);
+          break;
+        case WAITING:
+          waitingList.add(proc);
+          break;
+        case WAITING_TIMEOUT:
+          waitingTimeoutList.add(proc);
+          break;
+        case FAILED:
+          failedList.add(proc);
+          break;
+        case ROLLEDBACK:
+        case INITIALIZING:
+          String msg = "Unexpected " + proc.getState() + " state for " + proc;
+          LOG.error(msg);
+          throw new UnsupportedOperationException(msg);
+        default:
+          break;
+      }
+    }
 
     // 3. Check the waiting procedures to see if some of them can be added to runnable.
-    processWaitingProcedures(waitingList, runnableList);
-
+    waitingList.forEach(proc -> {
+      if (!proc.hasChildren()) {
+        // Normally, WAITING procedures should be waken by its children. But, there is a case that,
+        // all the children are successful and before they can wake up their parent procedure, the
+        // master was killed. So, during recovering the procedures from ProcedureWal, its children
+        // are not loaded because of their SUCCESS state. So we need to continue to run this WAITING
+        // procedure. But before executing, we need to set its state to RUNNABLE, otherwise, a
+        // exception will throw:
+        // Preconditions.checkArgument(procedure.getState() == ProcedureState.RUNNABLE,
+        // "NOT RUNNABLE! " + procedure.toString());
+        proc.setState(ProcedureState.RUNNABLE);
+        runnableList.add(proc);
+      } else {
+        proc.afterReplay(getEnvironment());
+      }
+    });
     // 4. restore locks
     restoreLocks();
 
     // 5. Push the procedures to the timeout executor
-    processWaitingTimeoutProcedures(waitingTimeoutList);
+    waitingTimeoutList.forEach(proc -> {
+      proc.afterReplay(getEnvironment());
+      timeoutExecutor.add(proc);
+    });
 
     // 6. Push the procedure to the scheduler
-    pushProceduresAfterLoad(runnableList, failedList);
+    failedList.forEach(scheduler::addBack);
+    runnableList.forEach(p -> {
+      p.afterReplay(getEnvironment());
+      if (!p.hasParent()) {
+        sendProcedureLoadedNotification(p.getProcId());
+      }
+      scheduler.addBack(p);
+    });
     // After all procedures put into the queue, signal the worker threads.
     // Otherwise, there is a race condition. See HBASE-21364.
     scheduler.signalAll();
@@ -691,11 +646,15 @@ public class ProcedureExecutor<TEnvironment> {
       worker.awaitTermination();
     }
 
-    // log the still active threads, ThreadGroup.destroy is deprecated in JDK17 and it is not
-    // necessary for us to must destroy it here, so we just do a check and log
-    if (threadGroup.activeCount() > 0) {
-      LOG.error("There are still active thread in group {}, see STDOUT", threadGroup);
-      threadGroup.list();
+    // Destroy the Thread Group for the executors
+    // TODO: Fix. #join is not place to destroy resources.
+    try {
+      threadGroup.destroy();
+    } catch (IllegalThreadStateException e) {
+      LOG.error("ThreadGroup {} contains running threads; {}: See STDOUT", this.threadGroup,
+        e.getMessage());
+      // This dumps list of threads on STDOUT.
+      this.threadGroup.list();
     }
 
     // reset the in-memory state for testing
@@ -876,10 +835,12 @@ public class ProcedureExecutor<TEnvironment> {
       return;
     }
 
-    Procedure<TEnvironment> proc =
-      new FailedProcedure<>(procId.longValue(), procName, procOwner, nonceKey, exception);
+    completed.computeIfAbsent(procId, (key) -> {
+      Procedure<TEnvironment> proc =
+        new FailedProcedure<>(procId.longValue(), procName, procOwner, nonceKey, exception);
 
-    completed.putIfAbsent(procId, new CompletedProcedureRetainer<>(proc));
+      return new CompletedProcedureRetainer<>(proc);
+    });
   }
 
   // ==========================================================================
@@ -1119,7 +1080,6 @@ public class ProcedureExecutor<TEnvironment> {
 
     // Create the rollback stack for the procedure
     RootProcedureState<TEnvironment> stack = new RootProcedureState<>();
-    stack.setRollbackSupported(proc.isRollbackSupported());
     rollbackStack.put(currentProcId, stack);
 
     // Submit the new subprocedures
@@ -1481,75 +1441,42 @@ public class ProcedureExecutor<TEnvironment> {
     }
   }
 
-  // Returning null means we have already held the execution lock, so you do not need to get the
-  // lock entry for releasing
-  private IdLock.Entry getLockEntryForRollback(long procId) {
-    // Hold the execution lock if it is not held by us. The IdLock is not reentrant so we need
-    // this check, as the worker will hold the lock before executing a procedure. This is the only
-    // place where we may hold two procedure execution locks, and there is a fence in the
-    // RootProcedureState where we can make sure that only one worker can execute the rollback of
-    // a RootProcedureState, so there is no dead lock problem. And the lock here is necessary to
-    // prevent race between us and the force update thread.
-    if (!procExecutionLock.isHeldByCurrentThread(procId)) {
-      try {
-        return procExecutionLock.getLockEntry(procId);
-      } catch (IOException e) {
-        // can only happen if interrupted, so not a big deal to propagate it
-        throw new UncheckedIOException(e);
-      }
+  /**
+   * Execute the rollback of the full procedure stack. Once the procedure is rolledback, the
+   * root-procedure will be visible as finished to user, and the result will be the fatal exception.
+   */
+  private LockState executeRollback(long rootProcId, RootProcedureState<TEnvironment> procStack) {
+    Procedure<TEnvironment> rootProc = procedures.get(rootProcId);
+    RemoteProcedureException exception = rootProc.getException();
+    // TODO: This needs doc. The root proc doesn't have an exception. Maybe we are
+    // rolling back because the subprocedure does. Clarify.
+    if (exception == null) {
+      exception = procStack.getException();
+      rootProc.setFailure(exception);
+      store.update(rootProc);
     }
-    return null;
-  }
 
-  private void executeUnexpectedRollback(Procedure<TEnvironment> rootProc,
-    RootProcedureState<TEnvironment> procStack) {
-    if (procStack.getSubprocs() != null) {
-      // comparing proc id in reverse order, so we will delete later procedures first, otherwise we
-      // may delete parent procedure first and if we fail in the middle of this operation, when
-      // loading we will find some orphan procedures
-      PriorityQueue<Procedure<TEnvironment>> pq =
-        new PriorityQueue<>(procStack.getSubprocs().size(),
-          Comparator.<Procedure<TEnvironment>> comparingLong(Procedure::getProcId).reversed());
-      pq.addAll(procStack.getSubprocs());
-      for (;;) {
-        Procedure<TEnvironment> subproc = pq.poll();
-        if (subproc == null) {
-          break;
-        }
-        if (!procedures.containsKey(subproc.getProcId())) {
-          // this means it has already been rolledback
-          continue;
-        }
-        IdLock.Entry lockEntry = getLockEntryForRollback(subproc.getProcId());
-        try {
-          cleanupAfterRollbackOneStep(subproc);
-          execCompletionCleanup(subproc);
-        } finally {
-          if (lockEntry != null) {
-            procExecutionLock.releaseLockEntry(lockEntry);
-          }
-        }
-      }
-    }
-    IdLock.Entry lockEntry = getLockEntryForRollback(rootProc.getProcId());
-    try {
-      cleanupAfterRollbackOneStep(rootProc);
-    } finally {
-      if (lockEntry != null) {
-        procExecutionLock.releaseLockEntry(lockEntry);
-      }
-    }
-  }
-
-  private LockState executeNormalRollback(Procedure<TEnvironment> rootProc,
-    RootProcedureState<TEnvironment> procStack) {
     List<Procedure<TEnvironment>> subprocStack = procStack.getSubproceduresStack();
     assert subprocStack != null : "Called rollback with no steps executed rootProc=" + rootProc;
 
     int stackTail = subprocStack.size();
     while (stackTail-- > 0) {
       Procedure<TEnvironment> proc = subprocStack.get(stackTail);
-      IdLock.Entry lockEntry = getLockEntryForRollback(proc.getProcId());
+      IdLock.Entry lockEntry = null;
+      // Hold the execution lock if it is not held by us. The IdLock is not reentrant so we need
+      // this check, as the worker will hold the lock before executing a procedure. This is the only
+      // place where we may hold two procedure execution locks, and there is a fence in the
+      // RootProcedureState where we can make sure that only one worker can execute the rollback of
+      // a RootProcedureState, so there is no dead lock problem. And the lock here is necessary to
+      // prevent race between us and the force update thread.
+      if (!procExecutionLock.isHeldByCurrentThread(proc.getProcId())) {
+        try {
+          lockEntry = procExecutionLock.getLockEntry(proc.getProcId());
+        } catch (IOException e) {
+          // can only happen if interrupted, so not a big deal to propagate it
+          throw new UncheckedIOException(e);
+        }
+      }
       try {
         // For the sub procedures which are successfully finished, we do not rollback them.
         // Typically, if we want to rollback a procedure, we first need to rollback it, and then
@@ -1599,59 +1526,15 @@ public class ProcedureExecutor<TEnvironment> {
         }
       }
     }
-    return LockState.LOCK_ACQUIRED;
-  }
-
-  /**
-   * Execute the rollback of the full procedure stack. Once the procedure is rolledback, the
-   * root-procedure will be visible as finished to user, and the result will be the fatal exception.
-   */
-  private LockState executeRollback(long rootProcId, RootProcedureState<TEnvironment> procStack) {
-    Procedure<TEnvironment> rootProc = procedures.get(rootProcId);
-    RemoteProcedureException exception = rootProc.getException();
-    // TODO: This needs doc. The root proc doesn't have an exception. Maybe we are
-    // rolling back because the subprocedure does. Clarify.
-    if (exception == null) {
-      exception = procStack.getException();
-      rootProc.setFailure(exception);
-      store.update(rootProc);
-    }
-
-    if (procStack.isRollbackSupported()) {
-      LockState lockState = executeNormalRollback(rootProc, procStack);
-      if (lockState != LockState.LOCK_ACQUIRED) {
-        return lockState;
-      }
-    } else {
-      // the procedure does not support rollback, so typically we should not reach here, this
-      // usually means there are code bugs, let's just wait all the subprocedures to finish and then
-      // mark the root procedure as failure.
-      LOG.error(HBaseMarkers.FATAL,
-        "Root Procedure {} does not support rollback but the execution failed"
-          + " and try to rollback, code bug?",
-        rootProc, exception);
-      executeUnexpectedRollback(rootProc, procStack);
-    }
-
-    IdLock.Entry lockEntry = getLockEntryForRollback(rootProc.getProcId());
-    try {
-      // Finalize the procedure state
-      LOG.info("Rolled back {} exec-time={}", rootProc,
-        StringUtils.humanTimeDiff(rootProc.elapsedTime()));
-      procedureFinished(rootProc);
-    } finally {
-      if (lockEntry != null) {
-        procExecutionLock.releaseLockEntry(lockEntry);
-      }
-    }
 
+    // Finalize the procedure state
+    LOG.info("Rolled back {} exec-time={}", rootProc,
+      StringUtils.humanTimeDiff(rootProc.elapsedTime()));
+    procedureFinished(rootProc);
     return LockState.LOCK_ACQUIRED;
   }
 
   private void cleanupAfterRollbackOneStep(Procedure<TEnvironment> proc) {
-    if (testing != null && testing.shouldKillBeforeStoreUpdateInRollback()) {
-      kill("TESTING: Kill BEFORE store update in rollback: " + proc);
-    }
     if (proc.removeStackIndex()) {
       if (!proc.isSuccess()) {
         proc.setState(ProcedureState.ROLLEDBACK);
@@ -1694,6 +1577,15 @@ public class ProcedureExecutor<TEnvironment> {
       LOG.error(HBaseMarkers.FATAL, "CODE-BUG: Uncaught runtime exception for " + proc, e);
     }
 
+    // allows to kill the executor before something is stored to the wal.
+    // useful to test the procedure recovery.
+    if (testing != null && testing.shouldKillBeforeStoreUpdate()) {
+      String msg = "TESTING: Kill before store update";
+      LOG.debug(msg);
+      stop();
+      throw new RuntimeException(msg);
+    }
+
     cleanupAfterRollbackOneStep(proc);
 
     return LockState.LOCK_ACQUIRED;
@@ -1803,6 +1695,9 @@ public class ProcedureExecutor<TEnvironment> {
         }
       }
 
+      // Add the procedure to the stack
+      procStack.addRollbackStep(procedure);
+
       // allows to kill the executor before something is stored to the wal.
       // useful to test the procedure recovery.
       if (
@@ -1820,24 +1715,7 @@ public class ProcedureExecutor<TEnvironment> {
       // Commit the transaction even if a suspend (state may have changed). Note this append
       // can take a bunch of time to complete.
       if (procedure.needPersistence()) {
-        // Add the procedure to the stack
-        // See HBASE-28210 on why we need synchronized here
-        boolean needUpdateStoreOutsideLock = false;
-        synchronized (procStack) {
-          if (procStack.addRollbackStep(procedure)) {
-            updateStoreOnExec(procStack, procedure, subprocs);
-          } else {
-            needUpdateStoreOutsideLock = true;
-          }
-        }
-        // this is an optimization if we do not need to maintain rollback step, as all subprocedures
-        // of the same root procedure share the same root procedure state, if we can only update
-        // store under the above lock, the sub procedures of the same root procedure can only be
-        // persistent sequentially, which will have a bad performance. See HBASE-28212 for more
-        // details.
-        if (needUpdateStoreOutsideLock) {
-          updateStoreOnExec(procStack, procedure, subprocs);
-        }
+        updateStoreOnExec(procStack, procedure, subprocs);
       }
 
       // if the store is not running we are aborting
@@ -2068,6 +1946,34 @@ public class ProcedureExecutor<TEnvironment> {
       scheduler.signalAll();
     }
 
+    /**
+     * Encapsulates execution of the current {@link #activeProcedure} for easy tracing.
+     */
+    private long runProcedure() throws IOException {
+      final Procedure<TEnvironment> proc = this.activeProcedure;
+      int activeCount = activeExecutorCount.incrementAndGet();
+      int runningCount = store.setRunningProcedureCount(activeCount);
+      LOG.trace("Execute pid={} runningCount={}, activeCount={}", proc.getProcId(), runningCount,
+        activeCount);
+      executionStartTime.set(EnvironmentEdgeManager.currentTime());
+      IdLock.Entry lockEntry = procExecutionLock.getLockEntry(proc.getProcId());
+      try {
+        executeProcedure(proc);
+      } catch (AssertionError e) {
+        LOG.info("ASSERT pid=" + proc.getProcId(), e);
+        throw e;
+      } finally {
+        procExecutionLock.releaseLockEntry(lockEntry);
+        activeCount = activeExecutorCount.decrementAndGet();
+        runningCount = store.setRunningProcedureCount(activeCount);
+        LOG.trace("Halt pid={} runningCount={}, activeCount={}", proc.getProcId(), runningCount,
+          activeCount);
+        this.activeProcedure = null;
+        executionStartTime.set(Long.MAX_VALUE);
+      }
+      return EnvironmentEdgeManager.currentTime();
+    }
+
     @Override
     public void run() {
       long lastUpdate = EnvironmentEdgeManager.currentTime();
@@ -2079,27 +1985,7 @@ public class ProcedureExecutor<TEnvironment> {
             continue;
           }
           this.activeProcedure = proc;
-          int activeCount = activeExecutorCount.incrementAndGet();
-          int runningCount = store.setRunningProcedureCount(activeCount);
-          LOG.trace("Execute pid={} runningCount={}, activeCount={}", proc.getProcId(),
-            runningCount, activeCount);
-          executionStartTime.set(EnvironmentEdgeManager.currentTime());
-          IdLock.Entry lockEntry = procExecutionLock.getLockEntry(proc.getProcId());
-          try {
-            executeProcedure(proc);
-          } catch (AssertionError e) {
-            LOG.info("ASSERT pid=" + proc.getProcId(), e);
-            throw e;
-          } finally {
-            procExecutionLock.releaseLockEntry(lockEntry);
-            activeCount = activeExecutorCount.decrementAndGet();
-            runningCount = store.setRunningProcedureCount(activeCount);
-            LOG.trace("Halt pid={} runningCount={}, activeCount={}", proc.getProcId(), runningCount,
-              activeCount);
-            this.activeProcedure = null;
-            lastUpdate = EnvironmentEdgeManager.currentTime();
-            executionStartTime.set(Long.MAX_VALUE);
-          }
+          lastUpdate = TraceUtil.trace(this::runProcedure, new ProcedureSpanBuilder(proc));
         }
       } catch (Throwable t) {
         LOG.warn("Worker terminating UNNATURALLY {}", this.activeProcedure, t);
